@@ -57,7 +57,7 @@ typedef struct load_step_req_struct {
 	slurmdb_cluster_rec_t *cluster;
 	bool local_cluster;
 	slurm_msg_t *req_msg;
-	List resp_msg_list;
+	list_t *resp_msg_list;
 } load_step_req_struct_t;
 
 typedef struct load_step_resp_struct {
@@ -315,8 +315,22 @@ static int _get_stepmgr_steps(void *x, void *arg)
 	slurm_msg_t req_msg;
 	slurm_msg_t_init(&req_msg);
 	slurm_msg_set_r_uid(&req_msg, slurm_conf.slurmd_user_id);
-	slurm_conf_get_addr(sji->stepmgr, &req_msg.address,
-			    req_msg.flags);
+
+	if (slurm_conf_get_addr(sji->stepmgr, &req_msg.address, req_msg.flags))
+	{
+		/*
+		 * The node isn't in the conf, see if the
+		 * controller has an address for it.
+		 */
+		slurm_node_alias_addrs_t *alias_addrs = NULL;
+		if (!slurm_get_node_alias_addrs(sji->stepmgr, &alias_addrs)) {
+			add_remote_nodes_to_conf_tbls(alias_addrs->node_list,
+						      alias_addrs->node_addrs);
+			slurm_free_node_alias_addrs(alias_addrs);
+			slurm_conf_get_addr(sji->stepmgr, &req_msg.address,
+					    req_msg.flags);
+		}
+	}
 
 	job_step_info_request_msg_t req_data = {0};
 	req_data.step_id.job_id = sji->job_id;
@@ -433,7 +447,7 @@ _load_fed_steps(slurm_msg_t *req_msg, job_step_info_response_msg_t **resp,
 	int pthread_count = 0;
 	pthread_t *load_thread = 0;
 	load_step_req_struct_t *load_args;
-	List resp_msg_list;
+	list_t *resp_msg_list;
 
 	*resp = NULL;
 
@@ -700,7 +714,7 @@ extern int slurm_job_step_stat(slurm_step_id_t *step_id,
 	slurm_msg_t req_msg;
 	list_itr_t *itr;
 	slurm_step_id_t req;
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	ret_data_info_t *ret_data_info = NULL;
 	int rc = SLURM_SUCCESS;
 	slurm_step_layout_t *step_layout = NULL;
@@ -738,7 +752,8 @@ extern int slurm_job_step_stat(slurm_step_id_t *step_id,
 	memcpy(&req, step_id, sizeof(req));
 	memcpy(&resp_out->step_id, step_id, sizeof(resp_out->step_id));
 
-	req_msg.protocol_version = use_protocol_ver;
+	req_msg.protocol_version = MIN(SLURM_PROTOCOL_VERSION,
+				       use_protocol_ver);
 	req_msg.msg_type = REQUEST_JOB_STEP_STAT;
 	req_msg.data = &req;
 
@@ -812,7 +827,7 @@ extern int slurm_job_step_get_pids(slurm_step_id_t *step_id,
 	slurm_msg_t req_msg;
 	slurm_step_id_t req;
 	list_itr_t *itr;
-	List ret_list = NULL;
+	list_t *ret_list = NULL;
 	ret_data_info_t *ret_data_info = NULL;
 	slurm_step_layout_t *step_layout = NULL;
 	job_step_pids_response_msg_t *resp_out;
