@@ -1330,17 +1330,6 @@ static bool _watch_loop(void)
 
 extern void *watch(void *arg)
 {
-#if HAVE_SYS_PRCTL_H
-	{
-		static char title[] = "watch";
-
-		if (prctl(PR_SET_NAME, title, NULL, NULL, NULL)) {
-			error("%s: cannot set process name to %s %m",
-			      __func__, title);
-		}
-	}
-#endif
-
 	slurm_mutex_lock(&mgr.mutex);
 
 	xassert(mgr.watch_thread == pthread_self());
@@ -1392,3 +1381,17 @@ extern void *watch(void *arg)
 
 	return NULL;
 }
+
+extern void *watch_thread(void *arg)
+{
+#if HAVE_SYS_PRCTL_H
+	static char title[] = "watch";
+
+	if (prctl(PR_SET_NAME, title, NULL, NULL, NULL)) {
+		error("%s: cannot set process name to %s %m",
+		      __func__, title);
+	}
+#endif
+
+	return watch(NULL);
+}
