@@ -88,7 +88,7 @@ strong_alias(jobacctinfo_create, slurm_jobacctinfo_create);
 strong_alias(jobacctinfo_destroy, slurm_jobacctinfo_destroy);
 
 typedef struct slurm_jobacct_gather_ops {
-	void (*poll_data) (List task_list, uint64_t cont_id, bool profile);
+	void (*poll_data) (list_t *task_list, uint64_t cont_id, bool profile);
 	int (*endpoll)    ();
 	int (*add_task)   (pid_t pid, jobacct_id_t *jobacct_id);
 } slurm_jobacct_gather_ops_t;
@@ -106,18 +106,17 @@ static const char *syms[] = {
 static slurm_jobacct_gather_ops_t ops;
 static plugin_context_t *g_context = NULL;
 static pthread_mutex_t g_context_lock = PTHREAD_MUTEX_INITIALIZER;
-static bool init_run = false;
+static plugin_init_t plugin_inited = PLUGIN_NOT_INITED;
 static pthread_mutex_t init_run_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_t watch_tasks_thread_id = 0;
 
 static int freq = 0;
-static List task_list = NULL;
+static list_t *task_list = NULL;
 static uint64_t cont_id = NO_VAL64;
 static pthread_mutex_t task_list_lock = PTHREAD_MUTEX_INITIALIZER;
 
 static bool jobacct_shutdown = true;
 static pthread_mutex_t jobacct_shutdown_mutex = PTHREAD_MUTEX_INITIALIZER;
-static bool plugin_polling = true;
 
 static slurm_step_id_t jobacct_step_id = {
 	.job_id = 0,
@@ -328,7 +327,7 @@ static bool _init_run_test(void)
 {
 	bool rc;
 	slurm_mutex_lock(&init_run_mutex);
-	rc = init_run;
+	rc = (plugin_inited == PLUGIN_INITED);
 	slurm_mutex_unlock(&init_run_mutex);
 	return rc;
 }
@@ -524,13 +523,15 @@ extern int jobacct_gather_init(void)
 	char    *plugin_type = "jobacct_gather";
 	int	retval=SLURM_SUCCESS;
 
-	if (slurmdbd_conf || (_init_run_test() && g_context))
-		return retval;
-
 	slurm_mutex_lock(&g_context_lock);
-	if (g_context)
+	if (plugin_inited)
 		goto done;
 
+	if (slurmdbd_conf || !slurm_conf.job_acct_gather_type) {
+		plugin_inited = PLUGIN_NOOP;
+		goto done;
+	}
+
 	g_context = plugin_context_create(plugin_type,
 					  slurm_conf.job_acct_gather_type,
 					  (void **) &ops, syms, sizeof(syms));
@@ -539,17 +540,12 @@ extern int jobacct_gather_init(void)
 		error("cannot create %s context for %s",
 		      plugin_type, slurm_conf.job_acct_gather_type);
 		retval = SLURM_ERROR;
-		goto done;
-	}
-
-	if (!xstrcasecmp(slurm_conf.job_acct_gather_type,
-			 "jobacct_gather/none")) {
-		plugin_polling = false;
+		plugin_inited = PLUGIN_NOT_INITED;
 		goto done;
 	}
 
 	slurm_mutex_lock(&init_run_mutex);
-	init_run = true;
+	plugin_inited = PLUGIN_INITED;
 	slurm_mutex_unlock(&init_run_mutex);
 
 	/* only print the warning messages if in the slurmctld */
@@ -560,10 +556,8 @@ extern int jobacct_gather_init(void)
 		warning("We will use a much slower algorithm with proctrack/pgid, use Proctracktype=proctrack/linuxproc or some other proctrack when using %s",
 			slurm_conf.job_acct_gather_type);
 
-	if (!xstrcasecmp(slurm_conf.accounting_storage_type,
-	                 ACCOUNTING_STORAGE_TYPE_NONE)) {
-		warning("Even though we are collecting accounting information you have asked for it not to be stored (%s). If this is not what you have in mind you will need to change it.",
-			ACCOUNTING_STORAGE_TYPE_NONE);
+	if (!slurm_conf.accounting_storage_type) {
+		warning("Even though we are collecting accounting information you have asked for it not to be stored (no AccountingStorageType set). If this is not what you have in mind you will need to change it.");
 	}
 
 done:
@@ -575,25 +569,32 @@ done:
 extern int jobacct_gather_fini(void)
 {
 	int rc = SLURM_SUCCESS;
+	static bool fini_ran = false;
 
 	slurm_mutex_lock(&g_context_lock);
-	if (g_context) {
-		slurm_mutex_lock(&init_run_mutex);
-		init_run = false;
-		slurm_mutex_unlock(&init_run_mutex);
+	if (fini_ran) {
+		slurm_mutex_unlock(&g_context_lock);
+		return SLURM_SUCCESS;
+	}
 
+	fini_ran = true;
+
+	if (g_context) {
 		if (watch_tasks_thread_id) {
 			slurm_mutex_unlock(&g_context_lock);
 			slurm_mutex_lock(&profile_timer->notify_mutex);
 			slurm_cond_signal(&profile_timer->notify);
 			slurm_mutex_unlock(&profile_timer->notify_mutex);
-			pthread_join(watch_tasks_thread_id, NULL);
+			slurm_thread_join(watch_tasks_thread_id);
 			slurm_mutex_lock(&g_context_lock);
 		}
 
 		rc = plugin_context_destroy(g_context);
 		g_context = NULL;
 	}
+	slurm_mutex_lock(&init_run_mutex);
+	plugin_inited = PLUGIN_NOT_INITED;
+	slurm_mutex_unlock(&init_run_mutex);
 	slurm_mutex_unlock(&g_context_lock);
 
 	return rc;
@@ -603,10 +604,10 @@ extern int jobacct_gather_startpoll(uint16_t frequency)
 {
 	int retval = SLURM_SUCCESS;
 
-	if (!plugin_polling)
-		return SLURM_SUCCESS;
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
 
-	xassert(g_context);
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	if (!_jobacct_shutdown_test()) {
 		error("jobacct_gather_startpoll: poll already started!");
@@ -636,7 +637,10 @@ extern int jobacct_gather_endpoll(void)
 {
 	int retval = SLURM_SUCCESS;
 
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
+
+	if (plugin_inited == PLUGIN_NOOP)
+		return SLURM_SUCCESS;
 
 	slurm_mutex_lock(&jobacct_shutdown_mutex);
 	jobacct_shutdown = true;
@@ -656,9 +660,9 @@ extern int jobacct_gather_add_task(pid_t pid, jobacct_id_t *jobacct_id,
 {
 	struct jobacctinfo *jobacct;
 
-	xassert(g_context);
+	xassert(plugin_inited != PLUGIN_NOT_INITED);
 
-	if (!plugin_polling)
+	if (plugin_inited == PLUGIN_NOOP)
 		return SLURM_SUCCESS;
 
 	if (_jobacct_shutdown_test())
@@ -695,7 +699,7 @@ error:
 
 extern jobacctinfo_t *jobacct_gather_stat_task(pid_t pid, bool update_data)
 {
-	if (!plugin_polling || _jobacct_shutdown_test())
+	if ((plugin_inited == PLUGIN_NOOP) || _jobacct_shutdown_test())
 		return NULL;
 
 	if (update_data)
@@ -704,7 +708,7 @@ extern jobacctinfo_t *jobacct_gather_stat_task(pid_t pid, bool update_data)
 	if (pid) {
 		struct jobacctinfo *jobacct = NULL;
 		struct jobacctinfo *ret_jobacct = NULL;
-		ListIterator itr = NULL;
+		list_itr_t *itr = NULL;
 
 		slurm_mutex_lock(&task_list_lock);
 		if (!task_list) {
@@ -734,9 +738,9 @@ extern jobacctinfo_t *jobacct_gather_stat_task(pid_t pid, bool update_data)
 extern jobacctinfo_t *jobacct_gather_remove_task(pid_t pid)
 {
 	struct jobacctinfo *jobacct = NULL;
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 
-	if (!plugin_polling)
+	if (plugin_inited == PLUGIN_NOOP)
 		return NULL;
 
 	/* poll data one last time before removing task
@@ -774,7 +778,7 @@ error:
 
 extern int jobacct_gather_set_proctrack_container_id(uint64_t id)
 {
-	if (!plugin_polling)
+	if (plugin_inited == PLUGIN_NOOP)
 		return SLURM_SUCCESS;
 
 	if (cont_id != NO_VAL64)
@@ -794,7 +798,7 @@ extern int jobacct_gather_set_proctrack_container_id(uint64_t id)
 extern int jobacct_gather_set_mem_limit(slurm_step_id_t *step_id,
 					uint64_t mem_limit)
 {
-	if (!plugin_polling)
+	if (plugin_inited == PLUGIN_NOOP)
 		return SLURM_SUCCESS;
 
 	if ((step_id->job_id == 0) || (mem_limit == 0)) {
@@ -813,7 +817,7 @@ extern int jobacct_gather_set_mem_limit(slurm_step_id_t *step_id,
 extern void jobacct_gather_handle_mem_limit(uint64_t total_job_mem,
 					    uint64_t total_job_vsize)
 {
-	if (!plugin_polling)
+	if (plugin_inited == PLUGIN_NOOP)
 		return;
 
 	if (jobacct_mem_limit)
@@ -840,7 +844,7 @@ extern jobacctinfo_t *jobacctinfo_create(jobacct_id_t *jobacct_id)
 	struct jobacctinfo *jobacct;
 	jobacct_id_t temp_id;
 
-	if (!plugin_polling)
+	if (plugin_inited == PLUGIN_NOOP)
 		return NULL;
 
 	jobacct = xmalloc(sizeof(struct jobacctinfo));
@@ -880,7 +884,7 @@ extern int jobacctinfo_setinfo(jobacctinfo_t *jobacct,
 	struct jobacctinfo *send = (struct jobacctinfo *) data;
 	buf_t *buffer = NULL;
 
-	if (!plugin_polling)
+	if (plugin_inited == PLUGIN_NOOP)
 		return SLURM_SUCCESS;
 
 	switch (type) {
@@ -955,7 +959,7 @@ extern int jobacctinfo_getinfo(
 	struct jobacctinfo *send = (struct jobacctinfo *) data;
 	char *buf = NULL;
 
-	if (!plugin_polling)
+	if (plugin_inited == PLUGIN_NOOP)
 		return SLURM_SUCCESS;
 
 	/* jobacct needs to be allocated before this is called.	*/
@@ -1013,7 +1017,8 @@ extern void jobacctinfo_pack(jobacctinfo_t *jobacct, uint16_t rpc_version,
 {
 	bool no_pack;
 
-	no_pack = (!plugin_polling && (protocol_type != PROTOCOL_TYPE_DBD));
+	no_pack = ((plugin_inited == PLUGIN_NOOP) &&
+		   (protocol_type != PROTOCOL_TYPE_DBD));
 
 	if (!jobacct || no_pack) {
 		pack8((uint8_t) 0, buffer);
@@ -1156,7 +1161,7 @@ unpack_error:
 
 extern void jobacctinfo_aggregate(jobacctinfo_t *dest, jobacctinfo_t *from)
 {
-	if (!plugin_polling)
+	if (plugin_inited == PLUGIN_NOOP)
 		return;
 
 	xassert(dest);
@@ -1204,7 +1209,7 @@ extern void jobacctinfo_2_stats(slurmdb_stats_t *stats, jobacctinfo_t *jobacct)
 	_jobacctinfo_2_stats_tres_usage(stats, jobacct);
 }
 
-extern long jobacct_gather_get_clk_tck()
+extern long jobacct_gather_get_clk_tck(void)
 {
 	long hertz = sysconf(_SC_CLK_TCK);
 
