@@ -52,15 +52,22 @@ static void _topo_add_dist(uint32_t *dist, int inx)
 	}
 }
 
+/*
+ * returns 1 if switch "i" is better fit
+ * returns -1 if switch "j" is better fit
+ * returns 0 if there is no better fit
+ */
 static int _topo_compare_switches(int i, int j, int rem_nodes,
 				  int *switch_node_cnt, int rem_cpus,
-				  uint32_t *switch_cpu_cnt)
+				  uint32_t *switch_cpu_cnt, bool *i_fit_out)
 {
 	while (1) {
 		bool i_fit = ((switch_node_cnt[i] >= rem_nodes) &&
-			      (switch_cpu_cnt[i] >= rem_cpus));
+			      ((int) switch_cpu_cnt[i] >= rem_cpus));
 		bool j_fit = ((switch_node_cnt[j] >= rem_nodes) &&
-			      (switch_cpu_cnt[j] >= rem_cpus));
+			      ((int) switch_cpu_cnt[j] >= rem_cpus));
+		*i_fit_out = i_fit;
+
 		if (i_fit && j_fit) {
 			if (switch_node_cnt[i] < switch_node_cnt[j])
 				return 1;
@@ -102,6 +109,7 @@ static void _topo_choose_best_switch(uint32_t *dist, int *switch_node_cnt,
 				     int rem_cpus, int i, int *best_switch)
 {
 	int tcs = 0;
+	bool i_fit = false;
 
 	if (*best_switch == -1 || dist[i] == INFINITE || !switch_node_cnt[i]) {
 		/*
@@ -113,8 +121,9 @@ static void _topo_choose_best_switch(uint32_t *dist, int *switch_node_cnt,
 	}
 
 	tcs = _topo_compare_switches(i, *best_switch, rem_nodes,
-				     switch_node_cnt, rem_cpus, switch_cpu_cnt);
-	if (((dist[i] < dist[*best_switch]) && (tcs >= 0)) ||
+				     switch_node_cnt, rem_cpus, switch_cpu_cnt,
+				     &i_fit);
+	if (((dist[i] < dist[*best_switch]) && i_fit) ||
 	    ((dist[i] == dist[*best_switch]) && (tcs > 0))) {
 		/*
 		 * If closer and fit request OR
@@ -130,7 +139,7 @@ static void _topo_choose_best_switch(uint32_t *dist, int *switch_node_cnt,
  */
 static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 {
-	List      *switch_gres = NULL;		/* available GRES on switch */
+	list_t **switch_gres = NULL;		/* available GRES on switch */
 	bitstr_t **switch_node_bitmap = NULL;	/* nodes on this switch */
 	int       *switch_node_cnt = NULL;	/* total nodes on switch */
 	int       *switch_required = NULL;	/* set if has required node */
@@ -140,9 +149,9 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 	bitstr_t  *best_nodes_bitmap  = NULL;	/* required+low prio nodes */
 	int i, j, rc = SLURM_SUCCESS;
 	int best_cpu_cnt = 0, best_node_cnt = 0, req_node_cnt = 0;
-	List best_gres = NULL;
+	list_t *best_gres = NULL;
 	switch_record_t *switch_ptr;
-	List node_weight_list = NULL;
+	list_t *node_weight_list = NULL;
 	topo_weight_info_t *nw = NULL;
 	list_itr_t *iter;
 	node_record_t *node_ptr;
@@ -291,7 +300,7 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 	 * Identify the highest level switch to be used.
 	 * Note that nodes can be on multiple non-overlapping switches.
 	 */
-	switch_gres        = xcalloc(switch_record_cnt, sizeof(List));
+	switch_gres = xcalloc(switch_record_cnt, sizeof(list_t *));
 	switch_node_bitmap = xcalloc(switch_record_cnt, sizeof(bitstr_t *));
 	switch_node_cnt    = xcalloc(switch_record_cnt, sizeof(int));
 	switch_required    = xcalloc(switch_record_cnt, sizeof(int));
@@ -333,7 +342,7 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 	if (top_switch_inx == -1) {
 		error("%pJ unable to identify top level switch",
 		       job_ptr);
-		rc = SLURM_ERROR;
+		rc = ESLURM_REQUESTED_TOPO_CONFIG_UNAVAILABLE;
 		goto fini;
 	}
 
@@ -341,7 +350,7 @@ static int _eval_nodes_dfly(topology_eval_t *topo_eval)
 	if (req_nodes_bitmap &&
 	    !bit_super_set(req_nodes_bitmap,
 			   switch_node_bitmap[top_switch_inx])) {
-		rc = SLURM_ERROR;
+		rc = ESLURM_REQUESTED_TOPO_CONFIG_UNAVAILABLE;
 		info("%pJ requires nodes that do not have shared network",
 		     job_ptr);
 		goto fini;
@@ -738,6 +747,22 @@ fini:
 	return rc;
 }
 
+static void _decrement_node_cnt(int num_nodes_taken, int switch_index,
+				int *switch_node_cnt)
+{
+	for (int i = switch_index; i >= 0; i = switch_record_table[i].parent) {
+		if (switch_node_cnt[i] <= num_nodes_taken) {
+			switch_node_cnt[i] = 0;
+		} else {
+			switch_node_cnt[i] -= num_nodes_taken;
+		}
+
+		/* end once we've reached root switch */
+		if (switch_record_table[i].parent == SWITCH_NO_PARENT)
+			break;
+	}
+}
+
 /* Allocate resources to job using a minimal leaf switch count */
 static int _eval_nodes_topo(topology_eval_t *topo_eval)
 {
@@ -754,14 +779,14 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 	bitstr_t *start_node_map = NULL;
 	int i, j, rc = SLURM_SUCCESS;
 	int best_cpu_cnt, best_node_cnt, req_node_cnt = 0;
-	List best_gres = NULL;
+	list_t *best_gres = NULL;
 	switch_record_t *switch_ptr;
-	List node_weight_list = NULL;
+	list_t *node_weight_list = NULL;
 	topo_weight_info_t *nw = NULL;
 	list_itr_t *iter;
 	node_record_t *node_ptr;
-	int64_t rem_max_cpus, start_rem_max_cpus;
-	int rem_cpus, start_rem_cpus, rem_nodes; /* remaining resources desired */
+	int64_t rem_max_cpus, start_rem_max_cpus = 0;
+	int rem_cpus, start_rem_cpus = 0, rem_nodes; /* remaining resources desired */
 	int min_rem_nodes;	/* remaining resources desired */
 	job_record_t *job_ptr = topo_eval->job_ptr;
 	job_details_t *details_ptr = job_ptr->details;
@@ -947,7 +972,7 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 	if (top_switch_inx == -1) {
 		log_flag(SELECT_TYPE, "%pJ unable to identify top level switch",
 			 job_ptr);
-		rc = SLURM_ERROR;
+		rc = ESLURM_REQUESTED_TOPO_CONFIG_UNAVAILABLE;
 		goto fini;
 	}
 
@@ -955,7 +980,7 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 	if (req_nodes_bitmap &&
 	    !bit_super_set(req_nodes_bitmap,
 			   switch_node_bitmap[top_switch_inx])) {
-		rc = SLURM_ERROR;
+		rc = ESLURM_REQUESTED_TOPO_CONFIG_UNAVAILABLE;
 		info("%pJ requires nodes that do not have shared network",
 		     job_ptr);
 		goto fini;
@@ -983,7 +1008,7 @@ static int _eval_nodes_topo(topology_eval_t *topo_eval)
 			goto fini;
 		}
 		if (topo_eval->max_nodes <= 0) {
-			rc = SLURM_ERROR;
+			rc = ESLURM_REQUESTED_TOPO_CONFIG_UNAVAILABLE;
 			log_flag(SELECT_TYPE, "%pJ requires nodes exceed maximum node limit",
 				 job_ptr);
 			goto fini;
@@ -1180,6 +1205,7 @@ try_again:
 
 	/* Add additional resources for already required leaf switches */
 	if (req_nodes_bitmap || req2_nodes_bitmap) {
+		int num_nodes_taken = 0;
 		for (i = 0; i < switch_record_cnt; i++) {
 			if (!switch_required[i] || !switch_node_bitmap[i] ||
 			    (switch_record_table[i].level != 0))
@@ -1197,6 +1223,7 @@ try_again:
 					avail_cpu_per_node[j] = 0;
 					continue;
 				}
+				num_nodes_taken++;
 				rem_nodes--;
 				min_rem_nodes--;
 				topo_eval->max_nodes--;
@@ -1211,6 +1238,9 @@ try_again:
 					goto fini;
 				}
 			}
+
+			_decrement_node_cnt(num_nodes_taken, i,
+					    switch_node_cnt);
 		}
 	}
 
@@ -1224,6 +1254,7 @@ try_again:
 	prev_rem_nodes = rem_nodes + 1;
 	while (1) {
 		int best_switch_inx = -1;
+
 		if (prev_rem_nodes == rem_nodes)
 			break; 	/* Stalled */
 		prev_rem_nodes = rem_nodes;
@@ -1275,6 +1306,8 @@ try_again:
 				goto fini;
 			}
 		}
+		_decrement_node_cnt(switch_node_cnt[best_switch_inx],
+				    best_switch_inx, switch_node_cnt);
 		switch_node_cnt[best_switch_inx] = 0;	/* Used all */
 	}
 	if ((min_rem_nodes <= 0) && (rem_cpus <= 0) &&
