@@ -119,6 +119,22 @@ time_t last_job_update = 0;
 bool time_limit_thread_shutdown = false;
 pthread_t time_limit_thread_id = 0;
 
+/* See _send_msg_maybe() in src/slurmctld/agent.c */
+static void _send_msg_maybe(slurm_msg_t *req)
+{
+	int fd = -1;
+
+	if ((fd = slurm_open_msg_conn(&req->address)) < 0) {
+		log_flag(NET, "%s: slurm_open_msg_conn(%pA): %m",
+			 __func__, &req->address);
+		return;
+	}
+
+	(void) slurm_send_node_msg(fd, req);
+
+	(void) close(fd);
+}
+
 static int _foreach_ret_data_info(void *x, void *arg)
 {
 	int rc;
@@ -137,6 +153,7 @@ static int _foreach_ret_data_info(void *x, void *arg)
 
 static void *_rpc_thread(void *data)
 {
+	bool srun_agent = false;
 	agent_arg_t *agent_arg_ptr = data;
 	slurm_msg_t msg;
 	slurm_msg_t_init(&msg);
@@ -148,9 +165,21 @@ static void *_rpc_thread(void *data)
 
 	slurm_msg_set_r_uid(&msg, agent_arg_ptr->r_uid);
 
+	srun_agent = ((msg.msg_type == SRUN_PING) ||
+		      (msg.msg_type == SRUN_JOB_COMPLETE) ||
+		      (msg.msg_type == SRUN_STEP_MISSING) ||
+		      (msg.msg_type == SRUN_STEP_SIGNAL) ||
+		      (msg.msg_type == SRUN_TIMEOUT) ||
+		      (msg.msg_type == SRUN_USER_MSG) ||
+		      (msg.msg_type == RESPONSE_RESOURCE_ALLOCATION) ||
+		      (msg.msg_type == SRUN_NODE_FAIL));
+
 	if (agent_arg_ptr->addr) {
 		msg.address = *agent_arg_ptr->addr;
-		if (slurm_send_only_node_msg(&msg)) {
+
+		if (msg.msg_type == SRUN_JOB_COMPLETE) {
+			_send_msg_maybe(&msg);
+		} else if (slurm_send_only_node_msg(&msg) && !srun_agent) {
 			error("failed to send message type %d/%s",
 			      msg.msg_type, rpc_num2string(msg.msg_type));
 		}
@@ -273,6 +302,11 @@ static void _init_stepd_stepmgr(void)
 			    NULL);
 }
 
+static void _on_sigalrm(conmgr_callback_args_t conmgr_args, void *arg)
+{
+	debug("Caught SIGALRM. Ignoring.");
+}
+
 static void _on_sigint(conmgr_callback_args_t conmgr_args, void *arg)
 {
 	info("Caught SIGINT. Shutting down.");
@@ -344,6 +378,7 @@ extern int main(int argc, char **argv)
 
 	conmgr_init(0, 0, (conmgr_callbacks_t) {0});
 
+	conmgr_add_work_signal(SIGALRM, _on_sigalrm, NULL);
 	conmgr_add_work_signal(SIGINT, _on_sigint, NULL);
 	conmgr_add_work_signal(SIGTERM, _on_sigterm, NULL);
 	conmgr_add_work_signal(SIGQUIT, _on_sigquit, NULL);
