@@ -68,10 +68,12 @@ struct squeue_parameters {
 	bool all_states;
 	bool array_flag;
 	bool detail_flag;
+	bool expand_patterns;
 	bool federation_flag;
 	int  iterate;
 	bool job_flag;
 	bool local_flag;
+	bool notme_flag;
 	bool sibling_flag;
 	bool start_flag;
 	bool step_flag;
@@ -82,8 +84,9 @@ struct squeue_parameters {
 	int  verbose;
 
 	char* accounts;
-	List clusters;
+	list_t *clusters;
 	uint32_t cluster_flags;
+	char *cluster_names;
 	char* format;
 	char* format_long;
 	char* jobs;
@@ -106,25 +109,43 @@ struct squeue_parameters {
 
 	uint32_t convert_flags;
 
-	List  account_list;
-	List  format_list;
-	List  job_list;
-	List  licenses_list;
-	List  name_list;
-	List  part_list;
-	List  qos_list;
-	List  state_list;
-	List  step_list;
-	List  user_list;
+	list_t *account_list;
+	list_t *format_list;
+	list_t *job_list;
+	list_t *licenses_list;
+	list_t *name_list;
+	list_t *part_list;
+	list_t *qos_list;
+	list_t *state_list;
+	list_t *step_list;
+	list_t *user_list;
 };
 
 extern struct squeue_parameters params;
 
+
+/* Flags for fmt_data_job_t, fmt_data_step_t */
+#define FMT_FLAG_HIDDEN		SLURM_BIT(0)
+
+typedef struct fmt_data_job {
+	char *name;		/* long format name */
+	char c;			/* short format character, prefixed by '%' */
+	int (*fn)(job_info_t *job, int width, bool right, char *suffix);
+	uint32_t flags;		/* FMT_FLAG_* */
+} fmt_data_job_t;
+
+typedef struct fmt_data_step {
+	char *name;		/* long format name */
+	char c;			/* short format character, prefixed by '%' */
+	int (*fn)(job_step_info_t *step, int width, bool right, char *suffix);
+	uint32_t flags;		/* FMT_FLAG_* */
+} fmt_data_step_t;
+
 extern void parse_command_line( int argc, char* *argv );
 extern int  parse_format( char* format );
 extern int  parse_long_format( char* format_long);
-extern void sort_job_list( List job_list );
-extern void sort_jobs_by_start_time( List job_list );
-extern void sort_step_list( List step_list );
+extern void sort_job_list(list_t *job_list);
+extern void sort_jobs_by_start_time(list_t *job_list);
+extern void sort_step_list(list_t *step_list);
 
 #endif
